/*

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is jslib team code.
The Initial Developer of the Original Code is jslib team.

Portions created by jslib team are
Copyright (C) 2000 jslib team.  All
Rights Reserved.

Original Author: Pete Collins <pete@mozdev.org>
Contributor(s):

*/

if(typeof(JS_LIB_LOADED)=='boolean')
{

function getClassIDFromInterface(aInterface)
{
  var C     = Components;
  var c     = C.classes;
  var list;

  try
  {
    for(list in c)
      if(typeof(C.Constructor(list, aInterface))=='function')
        return list;
  }

  catch(e){}

  return Components.results.NS_ERROR_NOT_IMPLEMENTED;
}

function getProgIDFromInterface(aInterface)
{
  return Components.results.NS_ERROR_NOT_IMPLEMENTED;
}

/****************************************************************
* getClassIDFromProgID(aProgID)                                 *       
*                                                               *
* aProgID is an argument of string ProgID (human readable name) *
* returns string CID on success, null on failure                *
*   Ex:                                                         *
*     var progID='@mozilla.org/file/local;1';                   *
*   getClassIDFromProgID(progID);                               *
*                                                               *
*   outputs: {2e23e220-60be-11d3-8c4a-000064657374}             *
****************************************************************/

function getClassIDFromProgID(aProgID)
{
  var retval=null;

  if(!aProgID)
  {
    debug("Please enter a valid Prog ID\n");
    throw Components.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
    return null;
  }

  if(typeof(Components.classes[aProgID])=="undefined")
    return (debug("invalid argument: '"+aProgID+"'\n"));
  
  if(Components.classes[aProgID].valid)
    retval=Components.classes[aProgID].number;

  return retval;
}

/****************************************************************
* getProgIDFromCID(aCID)                                        *       
*                                                               *
* aCID is an argument of string CID Class ID                    *
* returns string ProgID on success, null on failure             *
*   Ex:                                                         *
*       var cid='{2e23e220-60be-11d3-8c4a-000064657374}';       *
*       getClassIDFromProgID(cid);                              *
*                                                               *
*   outputs: @mozilla.org/file/local;1                          *
****************************************************************/

function getProgIDFromCID(aCID)
{

  var retval  = null;
  var C       = Components;
  var c       = C.classes;
  var cid     = C.classesByID;
  var res     = {};

  if(!aCID)
  {
    debug("Please enter a valid Class ID\n");
    throw C.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
    return null;
  }

  try
  {
    if(typeof(C.ID(aCID))=="undefined")
      retval=null;

    else
      if(C.ID(aCID).valid)
        retval=C.manager.CLSIDToContractID(Components.ID(aCID), res); 

  }

  catch(e){ retval=null; }

  try
  {

    if(typeof(cid[aCID])=="undefined")
      return (debug("invalid argument: '"+aCID+"'\n"));

    if(!retval)
      for(list in c)
        if(typeof(c[list])!="undefined")
          if(cid[aCID].equals(c[list]))
          {
            retval=list;
            break;
          }
  }

  catch(e){ retval=null; }

  return retval;
}

/****************************************************************
* getInterfaceFromProgID(aProgID)                               *
*                                                               *
* aProgID is an argument of string ProgID (human readable name) *
* returns an array of interfaces on success, null on failure    *
*   Ex:                                                         *
*       var progID='@mozilla.org/file/local;1';                 *
*       getInterfaceFromProgID(progID);                         *
*                                                               *
*   outputs: nsILocalFile,nsIFile                               *
****************************************************************/

function getInterfaceFromProgID(aProgID)
{

  var C     = Components;
  var iface = C.interfaces;
  var c     = C.classes;
  var res   = new Array();
  var list;
  var inst;

  if(!aProgID)
  {
    throw C.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
    return null;
  }
  
  if(typeof(c[aProgID])=='undefined')
    return (debug("invalid argument: '"+aProgID+"'\n"));

  inst=c[aProgID].getService();

  for(list in iface)
  {
    try
    {
      if(typeof(iface[list])!='undefined')
        if(typeof(inst.QueryInterface(iface[list]))=='object' && list!='nsISupports')
          res.push(list);
    }

    catch(e){}
  }

  return (res !="" ? res : null);

}

/****************************************************************
* getInterfaceFromCID(aCID)                                     *
*                                                               *
* aCID is an argument of string CID Class ID                    *
* returns array of interfaces on success, null on failure       *
*   Ex:                                                         *
*       var cid='{2e23e220-60be-11d3-8c4a-000064657374}';       *
*       getInterfaceFromCID(cid);                               *
*                                                               *
*   outputs: nsILocalFile,nsIFile                               *
****************************************************************/

function getInterfaceFromCID(aCID)
{

  var p;

  if(!aCID)
  {
    throw Components.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
    return null;
  }

  p=getProgIDFromCID(aCID);

  return  (typeof(p)!='undefined' ? getInterfaceFromProgID(p) : null);

}

function xpcMap(){ return Components.results.NS_ERROR_NOT_IMPLEMENTED; }

/****************************************************************
* lookUpError(aErrorNum)                                        *
*                                                               *
* aErrorNum is an argument of Components.results int            *
* returns string name of error on success, null on failure      *
*   Ex:                                                         *
*       var errNum=2147500033;                                  *
*       lookUpError(errNum);                                    *
*                                                               *
*   outputs: NS_ERROR_NOT_IMPLEMENTED                           *
****************************************************************/

function lookUpError(aErrorNum) {

  var r       = Components.results;
  var retval  = null;

  for(var list in r)
    if(aErrorNum==r[list])
      retval=list;

  return retval;
}

}

else
{
    dump("JSLIB library not loaded:\n"                                  +
         " \tTo load use: chrome://jslib/content/jslib.js\n"            +
         " \tThen: include('chrome://jslib/content/utils/xpcom.js');\n\n");
}
