/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is jslib team code.
The Initial Developer of the Original Code is jslib team.

Portions created by jslib team are
Copyright (C) 2000 jslib team.  All
Rights Reserved.

Original Author: Pete Collins <pete@mozdev.org>
Contributor(s): 

***/

if (typeof(JS_LIB_LOADED)=='boolean') {

  /****************** Globals **********************/

  const JS_ZIP_LOADED       = true;
  const JS_ZIP_FILE         = "zip.js";
  const JS_ZIP_CID          = "@mozilla.org/libjar/zip-reader;1";
  const JS_ZIP_I_ZIP_READER = "nsIZipReader";
  const JS_ZIP_FILE_CID     = "@mozilla.org/file/local;1";
  const JS_ZIP_I_LOCAL_FILE = "nsILocalFile";
  const JS_ZIP_FILE_INIT    = "initWithPath";
  
  function Zip () {}
  
  Zip.prototype = {
  
  ZipReader : new Components.Constructor(JS_ZIP_CID, JS_ZIP_I_ZIP_READER),
  ZipPath   : new Components.Constructor(JS_ZIP_FILE_CID, JS_ZIP_I_LOCAL_FILE, 
                                         JS_ZIP_FILE_INIT),
  DIRECTORY : 0x01, // 1
  perm      : 0755,
  OK        : void(null),
  FINISHED  : "*** FINISHED . . .  .",
  
  extract : function (aZipFile, aDest, aMsg) {
  
  if(arguments.length<2) {
    jslibError(null, ":no path defined", "NS_ERROR_INVALID_ARG", JS_ZIP_FILE+":extract");
    return null;
  }
  
  try {
    var dirpath   = aDest;
    var zipFile   = new this.ZipPath(aZipFile);
    var dest      = new this.ZipPath(aDest);
  
    if(!zipFile.exists()) {
      jslibDebug("zip.js:extract:ERROR: zip file doesn't exits . . .\n");
      jslibError(null, ":zip file doesn't exits . . .", 
                  "NS_ERROR_FILE_TARGET_DOES_NOT_EXIST", 
                  JS_ZIP_FILE+":extract");
      return null;
    }
  
    var dotIndex  = zipFile.leafName.lastIndexOf('.');
    var  ext      = (dotIndex >= 0) ? zipFile.leafName.substring(dotIndex) : "";
  
    var retval;
    switch (ext) {
  
    case ".jar":
      jslibDebug("\nzip.js:extract:CONFIRM: zip file has extension \""+ext+"\"\n\n");
      break;
  
    case ".zip":
      jslibDebug("\nzip.js:extract:CONFIRM: zip file has extension \""+ext+"\"\n\n");
      break;
  
    default:
      jslibDebug("zip.js:extract:ERROR: File doesn't look like a zip archive . . .\n");
      return null;
    }
  
    var zip   = new this.ZipReader();
    zip.init(zipFile);
    zip.open();
  
  } catch(e) { 
    jslibDebug("zip.js:extract:ERROR: "+ e.name + " " + e.message + "\n"); 
    return null;
  }
  
  var nsIZipEntry=Components.interfaces.nsIZipEntry;
  var entry;
  var newDir;
  
  try {
  
    if(!dest.exists() || !dest.isDirectory())
       dest.create( this.DIRECTORY, this.perm);
  
    var entries=zip.findEntries("*");
  
    jslibDebug("Extracting:\t"+zipFile.path+"\nTo:\t\t"+dest.path+" . . . \n\n");
  
    var destbase = new this.ZipPath(dirpath);
    while(entries.hasMoreElements()) {
      entry = entries.getNext().QueryInterface(nsIZipEntry);
      dest  = new this.ZipPath(dirpath);
      dest.setRelativeDescriptor(destbase, entry.name);
      newDir=dest.parent;
        if(!newDir.exists())
          newDir.create( this.DIRECTORY, this.perm);
      if(aMsg)
        aMsg.value="extracting: "+dest.leafName;
      jslibDebug("extracting: "+dest.leafName);
      zip.extract(entry.name, dest);
    }
  } catch(e) { 
    jslibDebug("zip.js:extract:ERROR: "+ e.name + " " + e.message + "\n"); 
    return null;
  }
  
  jslibDebug("\n\n"+this.FINISHED+"\n\n");
  
  if(aMsg)
    aMsg.value=this.FINISHED;
  
  return this.OK;
  
  }
} // END CLASS

jslibDebug('*** load: '+JS_ZIP_FILE+' OK');

} // END BLOCK JS_LIB_LOADED CHECK

// If jslib base library is not loaded, jslibDebug this error.
else {
    dump("JS_FILE library not loaded:\n"                                +
         " \tTo load use: chrome://jslib/content/jslib.js\n"            +
         " \tThen: include('chrome://jslib/content/io/file.js');\n\n");
}
